using System;
using System.Runtime.InteropServices;

namespace MPThread
{
    /// <summary>
    /// Kerry D. Wong
    /// http://www.kerrywong.com/PermaLink,guid,5ca6e18b-4088-4c54-8e3e-a688f18d498f.aspx
    /// </summary>
	public class PerformanceCounter
	{
		[DllImport("kernel32.dll")]
		private extern static short QueryPerformanceCounter(ref long x);
		[DllImport("kernel32.dll")]
		private extern static short QueryPerformanceFrequency(ref long x);

		private long freq = 0;
		private long counterStart = 0, counterEnd = 0;

		private long avgOverHead = 0;

		public long Frequency 
		{
			get 
			{
				if (freq == 0) 	QueryPerformanceFrequency(ref freq);
				return freq;
			}
		}

		public PerformanceCounter()	{
			int r = QueryPerformanceCounter(ref counterStart);

			if (r == 0)
				throw new Exception("High-resolution counter not supported.");

			Start();
			End();

			for (int i = 0 ; i < 100000 ; i++) {
				Start();
				avgOverHead += End();
			}

			avgOverHead = (long) ((double) avgOverHead / 100000.0);
		}

		public void Start() 
		{
			QueryPerformanceCounter(ref counterStart);
		}

		public long End()
		{
			QueryPerformanceCounter(ref counterEnd);

			return counterEnd - counterStart;
		}

		public double TimeElapsed(string unit) 
		{
			double diff = counterEnd - counterStart - avgOverHead;
			double seconds = diff / ((double) Frequency);

			switch (unit.Trim().ToUpper()) 
			{
				case "S": 
					return seconds;
				case "MS":
					return seconds * 1000.0;
				case "US":
					return seconds * 1000000.0;
				case "NS":
					return seconds * 1000000000.0;
				default:
					throw new Exception("Unit " + unit + " not supported.");
			}
		}
	}
}
